import requests
from requests import Request
from typing import Union, Optional

class ZhLyr(Request):
    '''
    `getter` lyrics class.
    '''
    method = 'GET'
    path: list = ['/lyrics/GetLyrics','/lyrics/GetLyrPrecisily']
    param: list = ['?q','?t','&a','&duration','&srt']
    Proofreading_Marks: list = ['ture','false']
    api:str = 'https://mtskyhazoki.pythonanywhere.com'
    send_request: requests.Session = requests.Session()
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def GetByTitle(cls, title:str,srt=None) -> dict:
        '''
        :param: `title`: title of the music to get lyrics from it.
        
        :param: `srt`: proofreading marks , if `true` he will return time as `srt` format.
        '''
        if srt == None:
            srt = cls.Proofreading_Marks[1]
            prepared_request = cls(
                url=fr'{cls.api}/{cls.path[0]}{cls.param[0]}={title}{cls.param[4]}={srt}',
                method=cls.method
            )
            prepped = cls.send_request.prepare_request(prepared_request)
            r = cls.send_request.send(prepped)
            return r.json()['lyrics']

    @classmethod
    def GetByDetails(cls, title:str,artist:str,duration:Optional[Union[str,int]]=None,srt=None) -> dict:
        '''
        :param `title`: title of the music to get lyrics from it.

        :param `artist`: artist of the music to get lyrics from it.

        :param `duration`: duration of the music to get lyrics from it.

        :param `srt`: proofreading marks , if `true` he will return time as `srt` format.
        '''
        if srt == None:
            srt = cls.Proofreading_Marks[1]
        if duration == None:
            duration = duration
        prepared_request = cls(
            url=fr'{cls.api}/{cls.path[1]}{cls.param[1]}={title}{cls.param[3]}={artist}{cls.param[3]}={duration}{cls.param[4]}={srt}',
            method=cls.method
        )
        prepped = cls.send_request.prepare_request(prepared_request)
        r = cls.send_request.send(prepped)
        return r.json()['lyrics']
    



from setuptools import setup , find_packages
with open('README.md','r') as file:
    setup(
        name = 'zhlyr',
        version = '3.8',
        packages = find_packages(),
        requires = ['requests','shazamio','shazam'],
        long_description =file.read(),
        long_description_content_type='text/markdown',
        description = 'Python library for music handling',
        author = 'Mtsky',
        author_email = 'secon2636@gmail.com',
        url = 'https://gaoc3.github.io/zhlyr/',
        license = 'MIT',
        classifiers=[
            "Development Status :: 1 - Planning",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3",
            "Operating System :: Unix",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ],
        keywords= ['lyrics','music','shazam','serialize','serializer','recognize'],
        python_requires=">=3.9"
    )



import json
from json import JSONDecodeError
from dataclasses import dataclass
from typing import Union, List, Dict, Any

@dataclass
class Serializer:
    '''
    A dataclass that can be serialized to JSON and deserialized from JSON.
    '''
    __data: Union[str, List, Dict[str, Any]]  # Making the field private

    def __post_init__(self):
        # Convert JSON string to dictionary if data is a string
        if isinstance(self.__data, str):
            self.__data = json.loads(self.__data)

    def __serializer(self, data):
        if isinstance(data, bool):
            return data
        elif isinstance(data, int):
            return data
        elif isinstance(data, float):
            return data
        elif isinstance(data, str):
            try:
                return json.loads(data)
            except JSONDecodeError:
                return data
        elif isinstance(data, list):
            return [self.__serializer(element) for element in data]
        elif isinstance(data, dict):
            return Serializer(data) 
        else:
            return data  # Return data unchanged for other types
    
    def __getattr__(self, item):
        # Allow direct access to JSON keys using dot notation
        if item in self.__data:
            return self.__serializer(self.__data[item])
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{item}'")

    def __repr__(self):
        json_str = ", ".join([f"{key}={self.__format_data(data)}" for key, data in self.__data.items()])
        return f"{self.__class__.__name__}(data=({json_str}))"

    def __format_data(self, data):
        if isinstance(data, list):
            return list(self.__format_data(element) for element in data)
        elif isinstance(data, dict):
            return Serializer(data)  # Use Serializer to create objects
        return data

